interface Event {
    type: string;
    action: (...args: any[]) => void;
}
declare class BaseApplication {
    canvas: HTMLCanvasElement;
    ctx: CanvasRenderingContext2D;
    _events: Map<string, Map<string, (...args: any[]) => void>>;
    _disposers: Set<() => void>;
    constructor(canvas: HTMLCanvasElement);
    on(event: Event): () => void;
    dispose(): void;
}

declare type Constructor<T = {}> = new (...args: any[]) => T;
interface ResizeEvent extends Event {
    type: 'resize';
    action: ({ app }: {
        app: ResizeApplication;
    }) => void;
}
declare type ResizeApplication = InstanceType<ReturnType<typeof withResize>>;
declare function withResize<T extends Constructor<BaseApplication>>(Base: T): {
    new (...args: any[]): {
        on(event: ResizeEvent): () => void;
        canvas: HTMLCanvasElement;
        ctx: CanvasRenderingContext2D;
        _events: Map<string, Map<string, (...args: any[]) => void>>;
        _disposers: Set<() => void>;
        dispose(): void;
    };
    mapKey: string;
} & T;
interface TickEvent extends Event {
    type: 'tick';
    action: ({ dt, app }: {
        dt: number;
        app: TickApplication;
    }) => void;
}
declare type TickApplication = InstanceType<ReturnType<typeof withTick>>;
declare function withTick<T extends Constructor<BaseApplication>>(Base: T): {
    new (...args: any[]): {
        "__#1@#last": number;
        "__#1@#tickID": number | null;
        isRunning: boolean;
        "__#1@#tick": () => void;
        start(): void;
        stop(): void;
        on(event: ResizeEvent | TickEvent): () => void;
        canvas: HTMLCanvasElement;
        ctx: CanvasRenderingContext2D;
        _events: Map<string, Map<string, (...args: any[]) => void>>;
        _disposers: Set<() => void>;
        dispose(): void;
    };
    mapKey: string;
} & T;

export { BaseApplication, Event, ResizeApplication, ResizeEvent, TickApplication, TickEvent, withResize, withTick };
